<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xlink="http://www.w3.org/1999/xlink" 
  xmlns:html="http://www.w3.org/HTML/1998/html4"
  xmlns:cals="http://www.infinity-loop.de/namespace/2006/upcast-cals"
  xmlns:mml="http://www.w3.org/1998/Math/MathML"
  xmlns:ilext="java:de.infinityloop.util.xslt.ExtensionFunctions"
  xmlns:ilf="http://www.infinity-loop.de/namespace/project/xslfunctions"
  xmlns:xd="http://www.pnp-software.com/XSLTdoc"
  xmlns:uci="http://www.infinity-loop.de/namespace/2006/upcast-internal"
  xmlns:saxon="http://saxon.sf.net/"
  xmlns:css="http://www.infinity-loop.de/namespace/2006/upcast-css"
  xmlns:csso="http://www.infinity-loop.de/namespace/2006/upcast-cssoverride"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:db="http://www.infinity-loop.de/namespace/internal/docbook45"
  exclude-result-prefixes="xd ilext uci ilf saxon css csso xlink xs html db mml cals" >

<!-- **************************************************
     XSLT stylesheet to transform a UPL-pre-processed upCast document tree
                     to DocBook 4.5.
                     
                     Root element: article
                     
                     
     DISCLAIMER:
       This stylesheet is provided AS IS with no warranty.
       Use completely at your own risk!
                       
     ******

     Author :  Christian Roth <roth@infinity-loop.de>
     Version:  2 (pre-2.x versions were used in upCast 5.x)
     
     ******
    
     History:
     2009-08-21 CR - added support for valign and align attributes on HTML table cells (td, th)
     
     ***************************************************
-->

  <!-- Parameterization. -->
  
  <!-- Parameters generated from the caller -->
  <xsl:param name="revisionmarkup">false</xsl:param>

  <!-- Handling of Generated Text -->
  <xsl:param name="usePIsForGentext">false</xsl:param>

  <!-- table model selection -->
  <xsl:param name="tablemodel">CALS</xsl:param>
  
  <!-- customizable parameters and settable via the UI by the user -->
  <!-- BLOCKS -->
  <!--
  <xsl:param name="blockquote">blockquote quote blockQuote BlockQuote Zitat</xsl:param>
  <xsl:param name="address">address Adresse</xsl:param>
  <xsl:param name="literallayout">literallayout literal</xsl:param>
  <xsl:param name="programlisting">listing programlisting</xsl:param>
  <xsl:param name="screen">screen</xsl:param>
  <xsl:param name="epigraph">epigraph</xsl:param>
  <xsl:param name="highlights">highlights Highlight</xsl:param>
  <xsl:param name="tip">tip Tipp</xsl:param>
  <xsl:param name="note">Note note Hinweis Anmerkung</xsl:param>
  <xsl:param name="caution">caution Caution Achtung Sicherheitshinweis</xsl:param>
  <xsl:param name="important">Important important wichtig Wichtig!</xsl:param>
  <xsl:param name="warning">Warnung warning</xsl:param>
  <xsl:param name="caption">caption figurecaption Beschriftung</xsl:param>
  -->
  <xsl:param name="personname"></xsl:param>

  <xsl:param name="blocks" select="(
    'blockquote',
    'address',
    'literallayout',
    'programlisting',
    'screen',
    'epigraph',
    'highlights',
    'tip',
    'note',
    'caution',
    'important',
    'warning',
    
    '-eos-'
    )"/>

<!-- INLINES -->
  <xsl:param name="tagclasses">'abbrev' 'acronym' 'emphasis' 'quote' 'trademark' 'citation' 'citetitle'
   'firstterm' 'glossterm' 'foreignphrase' 'wordasword' 'computeroutput' 'literal' 'markup' 'prompt' 'replaceable' 
   'sgmltag' 'userinput' 'subscript' 'superscript' 'accel' 'guibutton' 'guiicon' 'guilabel' 'guimenu' 'guimenuitem'
   'guisubmenu' 'keycap' 'keycode' 'keysym' 'mousebutton' 'action' 'classname' 'constant' 'errorcode' 'errorname'
   'errortype function' 'interface' 'msgtext' 'parameter' 'property' 'returnvalue' 'structfield' 'structname'
   'symbol' 'token' 'type' 'varname' 'application' 'command' 'envar' 'filename' 'medialabel' 'option' 'systemitem'
   'database' 'email' 'hardware' 'medialabel' 'optional'</xsl:param>


  <!-- Other options -->
  <xsl:param name="omitDefaultRole">true</xsl:param> <!-- if true, supresses writing role attributes when it is 'Normal' on <para>graphs and 'Default Paragraph Font' on <inline>s -->
  <xsl:param name="inlines.graphical">true</xsl:param> <!-- when true, also graphical-only inlines (manual formattings without a named style) are translated into emphasis elements with appropriate role attribute -->
  
  
  <!-- internal parameters -->
  <xsl:param name="excludeInAnnotation">table list par</xsl:param>
  <xsl:param name="ucBlockElements">document part section heading par list item footnote endnote table row cell entry tgroup tbody tr td th index toc annotation</xsl:param>
  <xsl:param name="subdocelements">'header' 'footer' 'footnote' 'endnote' 'annotation' 'subdoc' 'textbox'</xsl:param>
  <xsl:param name="mathmlnamespace">http://www.w3.org/1998/Math/MathML</xsl:param>
  
  <!-- Output method: XML -->
  <xsl:output method="xml" 
              indent="no"
              encoding="UTF-8"
              doctype-public="-//OASIS//DTD DocBook XML V4.5//EN"
              doctype-system="http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd"
              omit-xml-declaration="no"
  />

  <!-- DOCUMENT element -->
  <xsl:template match="uci:document">
    <article>
      <xsl:call-template name="id-attr"/>
      <xsl:call-template name="lang-attr"/>
      
      <title>
        <xsl:choose>
          <xsl:when test="uci:body/uci:par[@uci:rawclass='Title']">
            <xsl:apply-templates select="uci:body/uci:par[@uci:rawclass='Title']" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="uci:head/uci:documentinfo/uci:property[@uci:name='title']/@uci:value" />
          </xsl:otherwise>
        </xsl:choose>
      </title>
      <xsl:apply-templates select="uci:head"/>
      <xsl:apply-templates select="uci:body"/>
      <!-- We do not process uci:extflow here! Data from this part is fetched explicitly when needed. -->
    </article>
  </xsl:template>

  <!-- DOCUMENTINFO element -->
  <xsl:template match="uci:documentinfo">
    <articleinfo>
      <xsl:call-template name="id-attr"/>
      <xsl:call-template name="lang-attr"/>
      <xsl:choose>
        <xsl:when test="uci:property[@uci:name='title']">
          <title><xsl:value-of select="uci:property[@uci:name='title']/@uci:value"/></title>
        </xsl:when>
        <xsl:when test="/uci:document/uci:body/uci:par[@uci:rawclass='Title']">
          <title><xsl:value-of select="/uci:document/uci:body/uci:par[@uci:rawclass='Title']"/></title>
        </xsl:when>
        <xsl:otherwise>
          <title>Untitled Article</title>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="uci:property[not(@uci:name='title')]"/>
    </articleinfo>
  </xsl:template>


<!-- PROPERTY element -->
  <xsl:template match="uci:property">
    <xsl:choose>
    
      <!-- author -->
      <xsl:when test="@uci:name='author'">
        <xsl:if test="normalize-space(@uci:value)">
          <author>
            <xsl:copy-of select="ilext:parsePersonname(@uci:value)" />
          </author>
        </xsl:if>
      </xsl:when>
  
      <!-- operator -->
      <xsl:when test="@uci:name='operator'">
        <xsl:if test="normalize-space(@uci:value)">
          <othercredit role="operator">
            <xsl:copy-of select="ilext:parsePersonname(@uci:value)" />
          </othercredit>
        </xsl:if>
      </xsl:when>
  
      <!-- creationTime -->
      <xsl:when test="@uci:name='creationTime'">
        <date><xsl:value-of select="@uci:value" /></date>
      </xsl:when>
  
      <!-- editingMinutes -->
      <xsl:when test="@uci:name='editingMinutes'">
        <xsl:comment><xsl:text>Editing minutes: </xsl:text><xsl:value-of select="@uci:value" /></xsl:comment>
      </xsl:when>
  
      <!-- numberOfPages -->
      <xsl:when test="@uci:name='numberOfPages'">
        <xsl:comment><xsl:text>Original number of pages: </xsl:text><xsl:value-of select="@uci:value" /></xsl:comment>
      </xsl:when>
  
      <!-- numberOfWords -->
      <xsl:when test="@uci:name='numberOfWords'">
        <xsl:comment><xsl:text>Original number of words: </xsl:text><xsl:value-of select="@uci:value" /></xsl:comment>
      </xsl:when>
  
      <!-- numberOfChars -->
      <xsl:when test="@uci:name='numberOfChars'">
        <xsl:comment><xsl:text>Original number of characters: </xsl:text><xsl:value-of select="@uci:value" /></xsl:comment>
      </xsl:when>
  
      <!-- numberOfCharsWS -->
      <xsl:when test="@uci:name='numberOfCharsWS'">
        <xsl:comment><xsl:text>Original number of characters (incl. whitespace): </xsl:text><xsl:value-of select="@uci:value" /></xsl:comment>
      </xsl:when>
  
      <!-- revisionTime -->
      <xsl:when test="@uci:name='revisionTime'">
        <revhistory>
          <revision>
            <revnumber><xsl:text>1</xsl:text></revnumber>
            <date><xsl:value-of select="@uci:value" /></date>
          </revision>
        </revhistory>
      </xsl:when>
  
      <!-- lastPrintTime -->
      <xsl:when test="@uci:name='lastPrintTime'">
        <printhistory>
          <para><xsl:text>Last printed: </xsl:text><xsl:value-of select="@uci:value"/></para>
        </printhistory>
      </xsl:when>
  
      <!-- company -->
      <xsl:when test="@uci:name='company'">
        <corpname><xsl:value-of select="@uci:value" /></corpname>
      </xsl:when>
  
      <!-- ..unknown.. : write as comment -->
      <xsl:otherwise>
        <xsl:comment><xsl:value-of select="@uci:name" /><xsl:text>: </xsl:text><xsl:value-of select="@uci:value" /></xsl:comment>
      </xsl:otherwise>
  
    </xsl:choose>
  </xsl:template>
  

  <!-- PART element: skipped for article -->
  <xsl:template match="uci:part">
    <xsl:apply-templates />
  </xsl:template>

  
  <!-- SECTION elements -->
  <xsl:template match="uci:section">
    <section>
      <title><xsl:apply-templates select="uci:par[@uci:heading-level=current()/@uci:level]/node()" /></title>
      <xsl:choose>
        <xsl:when test="*[self::section or not(@uci:heading-level) or (@uci:heading-level ne current()/@uci:level)]">
          <xsl:apply-templates select="*[self::section or not(@uci:heading-level) or (@uci:heading-level ne current()/@uci:level)]" />
        </xsl:when>
        <xsl:otherwise>
          <!-- write empty paragraph to be valid -->
          <para><xsl:comment>empty para to satisfy content model</xsl:comment></para>
        </xsl:otherwise>
      </xsl:choose>
    </section>
  </xsl:template>


  <!-- PAR element -->
  <xsl:template match="uci:par[(@uci:heading-level cast as xs:integer)=0 and not(@db:caption)]">
    <!-- pull out before: contained textboxes -->
    <xsl:apply-templates select="descendant::uci:contentref[@uci:targettype='box']" mode="textbox"/>
    
    <para>
      <xsl:if test="($omitDefaultRole = 'false') or (@uci:rawclass ne 'Normal')">
        <xsl:attribute name="role"><xsl:value-of select="@uci:class" /></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates />
    </para>
  
    <!-- pull out after: INDEX -->
    <xsl:if test="descendant::uci:gentext[@uci:type='INDEX']">
      <index><xsl:comment> Index is to be generated here by output processing system </xsl:comment></index>
    </xsl:if>
  </xsl:template>

  <xsl:template match="uci:par[@db:caption]" mode="caption">
    <!-- pull out before: contained textboxes -->
    <xsl:apply-templates select="descendant::uci:contentref[@uci:targettype='box']" mode="textbox"/>
    
    <para>
      <xsl:if test="($omitDefaultRole = 'false') or (@uci:rawclass ne 'Normal')">
        <xsl:attribute name="role"><xsl:value-of select="@uci:class" /></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates />
    </para>
  
  </xsl:template>

  <xsl:template match="uci:par[not(@db:caption) and (@uci:heading-level cast as xs:integer) gt 0]">
    <!-- pull out before: contained textboxes -->
    <xsl:apply-templates select="descendant::uci:contentref[@uci:targettype='box']" mode="textbox"/>
    
    <para>
      <xsl:if test="($omitDefaultRole = 'false') or (@uci:rawclass ne 'Normal')">
        <xsl:attribute name="role"><xsl:value-of select="@uci:class" /></xsl:attribute>
      </xsl:if>
      <xsl:apply-templates />
    </para>
  
  </xsl:template>


  <!-- INLINE element -->
  <xsl:template match="uci:inline">
    
    <!-- Flag if this inline is a semantic DB element or just a style override -->
    <xsl:variable name="isElem" as="xs:boolean"><xsl:value-of select="contains( $tagclasses, concat( &quot;&apos;&quot;, @uci:rawclass, &quot;&apos;&quot;) ) and @uci:rawclass!=''" /></xsl:variable>
    <xsl:choose>
  
      <xsl:when test="contains( $personname, concat( &quot;'&quot;, @uci:rawclass, &quot;'&quot;) )">
        <xsl:variable name="name" select="string()" />
        <xsl:copy-of select="ilext:parsePersonname($name)" />
      </xsl:when>
  
      <xsl:when test="$isElem">
        <xsl:element name="{@uci:rawclass}">
          <xsl:call-template name="lang-attr"/>
          <xsl:apply-templates />
        </xsl:element>
      </xsl:when>
      
      <!-- sub, super -->
      <xsl:when test="(@csso:vertical-align!='baseline' and @csso:vertical-align!='sub') and (not( starts-with( @csso:vertical-align, '-' )) or @csso:vertical-align='super')">
        <superscript>
          <xsl:call-template name="lang-attr"/>
          <xsl:apply-templates select="child::node()[not(self::uci:contentref[@uci:targettype='footnote' or @uci:targettype='endnote']) or self::uci:image]"/> <!-- no footnotes, endnotes, images in superscript elements! -->
        </superscript>
        <xsl:apply-templates select="child::uci:contentref[@uci:targettype='footnote' or @uci:targettype='endnote'] | child::uci:image" />
      </xsl:when>
      <xsl:when test="starts-with( @csso:vertical-align, '-' ) or (@csso:vertical-align='sub')">
        <subscript>
          <xsl:call-template name="lang-attr"/>
          <xsl:apply-templates select="child::node()[not(self::uci:contentref[@uci:targettype='footnote' or @uci:targettype='endnote']) or self::uci:image]"/> <!-- see above -->
        </subscript>
        <xsl:apply-templates select="child::uci:contentref[@uci:targettype='footnote' or @uci:targettype='endnote'] | child::uci:image" />
      </xsl:when>
  
      <!-- italic -->
      <xsl:when test="@csso:font-style='italic' and $inlines.graphical = 'true'">
        <emphasis>
          <xsl:call-template name="lang-attr"/>
          <xsl:apply-templates />
        </emphasis>
      </xsl:when>

      <!-- bold -->
      <xsl:when test="@csso:font-weight='bold' and $inlines.graphical = 'true'">
        <emphasis role="bold">
          <xsl:call-template name="lang-attr"/>
          <xsl:apply-templates />
        </emphasis>
      </xsl:when>
      
      <xsl:otherwise>
        <xsl:choose>
          <xsl:when test="((@uci:class and (@uci:class ne 'Hyperlink')) or (@uci:rawclass and (@uci:rawclass ne 'Hyperlink'))) or @xml:lang">
            <phrase>
              <xsl:if test="@uci:class">
                <xsl:if test="($omitDefaultRole = 'false') or (@uci:rawclass ne 'Default Paragraph Font')">
                  <xsl:attribute name="role"><xsl:value-of select="@uci:class"/></xsl:attribute>
                </xsl:if>
              </xsl:if>
              <xsl:call-template name="lang-attr"/>
              <xsl:apply-templates />
            </phrase>
          </xsl:when>
          <xsl:otherwise>
            <!-- skip semantically irrelevant inlines -->
            <xsl:apply-templates />
          </xsl:otherwise>
        </xsl:choose>
      </xsl:otherwise>
  
    </xsl:choose>
  </xsl:template>

  <xsl:template match="uci:inline[@csso:ilx-text-visibility='none']">
    <!-- discard hidden content as we cannot mark that up in DocBook, but make sure we include elements like e.g. contentref -->
    <xsl:apply-templates select="descendant::uci:contentref"/>
  </xsl:template>


  <!-- LIST element -->
  <xsl:template match="uci:list">
    <xsl:variable name="lt"><xsl:value-of select="@css:list-style-type"/></xsl:variable>
    
    <!-- itemizedlist -->
    <xsl:choose>
      <xsl:when test="($lt='disc') or ($lt='square') or ($lt='circle') or ($lt='hyphen')">
        <itemizedlist mark="{$lt}">
          <xsl:attribute name="spacing">
            <xsl:choose>
              <xsl:when test="starts-with(@css:ilx-marker-offset, '-')"> <!-- negative indent = normal list! -->
                <xsl:text>normal</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>compact</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
          <xsl:apply-templates />
        </itemizedlist>
      </xsl:when>
      
      <!-- orderedlist -->
      <xsl:otherwise>
        <xsl:variable name="numeration" as="xs:string">
          <xsl:choose>
            <xsl:when test="$lt eq 'lower-roman'">lowerroman</xsl:when>
            <xsl:when test="$lt eq 'lower-alpha' or $lt eq 'lower-greek'">loweralpha</xsl:when>
            <xsl:when test="$lt eq 'upper-alpha'">upperalpha</xsl:when>
            <xsl:when test="$lt eq 'upper-roman'">upperroman</xsl:when>
            <xsl:otherwise>arabic</xsl:otherwise>
          </xsl:choose>
        </xsl:variable>
        
        <orderedlist numeration="{$numeration}">
          <xsl:if test="(@csso:ilx-list-numbering-absolute cast as xs:integer?) gt 1">
            <xsl:attribute name="continuation">continues</xsl:attribute>
          </xsl:if>
          <xsl:if test="string-length( string-join( tokenize( @csso:ilx-marker-format, '[^%]+' ), '' ) ) gt 1"> <!-- This means we have two placeholders in the format, i.e. we most probably inherit numeration. -->
            <xsl:attribute name="inheritnum">inherit</xsl:attribute>
          </xsl:if>
          <xsl:attribute name="spacing">
            <xsl:choose>
              <xsl:when test="starts-with(@css:ilx-marker-offset, '-')"> <!-- negative indent = normal list! -->
                <xsl:text>normal</xsl:text>
              </xsl:when>
              <xsl:otherwise>
                <xsl:text>compact</xsl:text>
              </xsl:otherwise>
            </xsl:choose>
          </xsl:attribute>
          <xsl:apply-templates />
        </orderedlist>

      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- ITEM element -->
  <xsl:template match="uci:item">
    <listitem>
    	<xsl:choose>
    		<xsl:when test="count( child::* ) = 0">
    			<para><xsl:comment>we need to create at least an empty para to make the document valid</xsl:comment></para>
    		</xsl:when>
    		<xsl:otherwise>
  		    <xsl:apply-templates />
    		</xsl:otherwise>
    	</xsl:choose>
    </listitem>
  </xsl:template>


  <!-- BLOCKs -->
  <xsl:template match="uci:block[exists( index-of( $blocks, @uci:type) )]">
    <xsl:element name="{@uci:type}">
      <xsl:apply-templates/>
    </xsl:element>
  </xsl:template>
  
  <!-- Special Block: literallayout -->
  <xsl:template match="uci:block[@uci:type='literallayout']" priority="1">
    <xsl:element name="{@uci:type}">
      <xsl:apply-templates select="*[not(self::uci:par)]"/>
    </xsl:element>
  </xsl:template>


  <!-- IMAGE element with caption -->
  <xsl:template match="uci:block[@uci:type='captioned-image']">
    <mediaobject>
      <xsl:apply-templates select="descendant::uci:image" mode="image"/>
      <caption><xsl:apply-templates select="uci:par[@db:caption]" mode="caption"/></caption>
    </mediaobject>
  </xsl:template>
  
  <!-- IMAGE element without caption -->
  <xsl:template match="uci:image">
    <inlinemediaobject>
      <xsl:apply-templates select="." mode="image"/>
    </inlinemediaobject>
  </xsl:template>
  
  <xsl:template match="uci:image" mode="image">
    <imageobject>
      <xsl:element name="imagedata">
        <xsl:if test="@uci:width">
          <xsl:attribute name="width"><xsl:value-of select="@uci:width" /></xsl:attribute>
        </xsl:if>
        <xsl:if test="@uci:width_px">
          <xsl:attribute name="width"><xsl:value-of select="@uci:width_px" /><xsl:text>px</xsl:text></xsl:attribute>
        </xsl:if>
        <xsl:if test="@uci:height">
          <xsl:attribute name="depth"><xsl:value-of select="@uci:height" /></xsl:attribute>
        </xsl:if>
        <xsl:if test="@uci:height_px">
          <xsl:attribute name="depth"><xsl:value-of select="@uci:height_px" /><xsl:text>px</xsl:text></xsl:attribute>
        </xsl:if>
        <xsl:attribute name="fileref"><xsl:value-of select="@uci:href" /></xsl:attribute>
        <xsl:if test="@csso:float and @csso:float ne 'none'">
          <xsl:attribute name="align"><xsl:value-of select="@csso:float" /></xsl:attribute>
        </xsl:if>
      </xsl:element>
    </imageobject>
    <xsl:if test="@uci:alttext">
      <textobject>
        <para><xsl:value-of select="@uci:alttext" /></para>
      </textobject>
    </xsl:if>
  </xsl:template>


  <!-- REFERENCE element -->
  <xsl:template match="uci:body//uci:reference">
    <xref>
      <xsl:attribute name="linkend"><xsl:value-of select="ilext:makeValidXML_ID(@uci:rawtarget)" /></xsl:attribute>
    </xref>
  </xsl:template>


  <!-- FOOTNOTE and ENDNOTE -->
  <!-- endnotes are not supported by docbook. We therefore present them as being footnotes with the role "endnote". -->
  <xsl:template match="uci:contentref[@uci:role='definition' and (@uci:targettype='footnote' or @uci:targettype='endnote')]">
    <footnote>
      <xsl:attribute name="id" select="@uci:idref"/>
      <xsl:if test="@uci:targettype = 'endnote'">
        <xsl:attribute name="role">endnote</xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/uci:content/*" /> <!-- =the definition -->
    </footnote>
  </xsl:template>

  <xsl:template match="uci:footnote | uci:endnote">
    <xsl:apply-templates select="uci:content/node()"/>
  </xsl:template>

  <!-- a second reference to a footnote or endnote -->
  <xsl:template match="uci:contentref[@uci:role='reference' and (@uci:targettype='footnote' or @uci:targettype='endnote')]">
    <footnoteref linkend="{@uci:idref}">
      <xsl:if test="@uci:targettype = 'endnote'">
        <xsl:attribute name="role">endnote</xsl:attribute>
      </xsl:if>
    </footnoteref>
  </xsl:template>


  <!-- TEXTBOX -->
  <xsl:template match="uci:contentref[@uci:targettype='box']" mode="textbox">
    <sidebar>
      <xsl:apply-templates select="/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/node()"/>
    </sidebar>
  </xsl:template>
  
  <xsl:template match="uci:contentref[@uci:targettype='box']">
    <!-- exclude, handled explicitly elsewhere -->
  </xsl:template>
  

  <!-- ANNOTATION -->
  <xsl:template match="uci:contentref[@uci:role='definition' and (@uci:targettype='annotation')]">
    <remark>
      <xsl:attribute name="id" select="@uci:idref"/>
      <xsl:if test="/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/@uci:authorname">
        <author>
          <xsl:copy-of select="ilext:parsePersonname(/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/@uci:authorname)" />
        </author>
      </xsl:if>
      <xsl:if test="/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/@uci:authorid">
        <authorinitials>
          <xsl:value-of select="/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/@uci:authorid" />
        </authorinitials>
      </xsl:if>
      <!-- remarks are inline elements! We therefore flatten the content to inline content only. -->
      <xsl:apply-templates select="/uci:document/uci:extflow//uci:*[@uci:id=current()/@uci:idref]/uci:content/descendant::node()[not(contains($excludeInAnnotation, local-name()))]"/>
    </remark>
  </xsl:template>


  <!-- INDEXTARGET -->
  <xsl:template match="uci:contentref[@uci:targettype='indextarget']">
    <xsl:variable name="entry">
      <xsl:sequence select="/uci:document/uci:extflow//*[@uci:id=current()/@uci:idref]"/>
    </xsl:variable>
    
    <indexterm>
      <xsl:attribute name="significance" select="$entry/uci:indextarget/@uci:significance"></xsl:attribute>
      <xsl:if test="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='1']">
        <primary><xsl:value-of select="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='1']" /></primary>
      </xsl:if>
      <xsl:if test="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='2']">
        <secondary><xsl:value-of select="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='2']" /></secondary>
      </xsl:if>
      <xsl:if test="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='3']">
        <tertiary><xsl:value-of select="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='3']" /></tertiary>
      </xsl:if>
      <xsl:if test="$entry/uci:indextarget/uci:see">
        <see><xsl:apply-templates select="$entry/uci:indextarget/uci:see/node()" /></see>
      </xsl:if>
    </indexterm>
    <xsl:if test="
      ($entry/uci:indextarget/uci:indexentry[@uci:indexlevel='4']) 
      or ($entry/uci:indextarget/uci:indexentry[@uci:indexlevel='5'])
      or ($entry/uci:indextarget/uci:indexentry[@uci:indexlevel='6'])
      or ($entry/uci:indextarget/uci:indexentry[@uci:indexlevel='7'])
      or ($entry/uci:indextarget/uci:indexentry[@uci:indexlevel='8'])
      or ($entry/uci:indextarget/uci:indexentry[@uci:indexlevel='9'])
      ">
      <xsl:message>index term '<xsl:value-of select="$entry/uci:indextarget/uci:indexentry[@uci:indexlevel='1']" />' has more than the 3 levels DocBook allows</xsl:message>
    </xsl:if> 
  </xsl:template>
  

  <!-- INSERTED element -->
  <xsl:template match="uci:inserted">
    <xsl:choose>
      <xsl:when test="$revisionmarkup='true'">
        <phrase revisionflag="added">
          <xsl:apply-templates />
        </phrase>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  
  <!-- DELETED element -->
  <xsl:template match="uci:deleted">
    <xsl:choose>
      <xsl:when test="$revisionmarkup='true'">
        <phrase revisionflag="deleted">
          <xsl:apply-templates />
        </phrase>
      </xsl:when>
      <xsl:otherwise>
        <!-- content gets completely stripped from document -->
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- GENTEXT element -->
  <xsl:template match="uci:gentext">
    <xsl:choose>
    
      <!-- index location -->
      <xsl:when test="(@uci:type = 'INDEX')">
        <xsl:if test="count(ancestor::uci:par)=0"> <!-- not in a par element -->
          <index><xsl:comment> Index is to be generated here by processing system </xsl:comment></index>
        </xsl:if>
      </xsl:when>
    
      <xsl:otherwise>
        <xsl:if test="(@uci:type != 'upcast-FOOTNOTENUMBER') and (@uci:type != 'upcast-HEADINGNUMBER') and (@uci:type != 'upcast-ANNOTATIONNUMBER')">
          <xsl:choose>
            <xsl:when test="$usePIsForGentext='true'">
              <xsl:processing-instruction name="uc-gentext">
                <xsl:text>type="</xsl:text><xsl:value-of select="@uci:type"/><xsl:text>" data="</xsl:text><xsl:value-of select="ilf:quoteForAttribute(@uci:data)"/><xsl:text>" value="</xsl:text><xsl:value-of select="descendant::text()"/><xsl:text>"</xsl:text>
              </xsl:processing-instruction>
            </xsl:when>
            <xsl:otherwise>
              <phrase>
                <xsl:attribute name="role">
                  <xsl:text>GEN_</xsl:text><xsl:value-of select="@uci:type" />
                </xsl:attribute>
                <xsl:apply-templates />
              </phrase>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
      </xsl:otherwise>
      
    </xsl:choose>
  </xsl:template>


  <!-- TARGET element -->
  <xsl:template match="uci:target">
    <anchor>
      <xsl:attribute name="id"><xsl:value-of select="ilext:makeValidXML_ID(@uci:rawtarget)" /></xsl:attribute>
    </anchor>
  </xsl:template>


  <!-- LINK element -->
  <xsl:template match="uci:link">
    <xsl:choose>
      <xsl:when test="starts-with(@uci:href, '#')">
        <link>
          <xsl:attribute name="linkend"><xsl:value-of select="ilext:makeValidXML_ID(@uci:rawtarget)" /></xsl:attribute>
          <xsl:apply-templates />
        </link>
      </xsl:when>
      <xsl:otherwise>
        <ulink>
          <xsl:attribute name="url"><xsl:value-of select="@uci:href" /></xsl:attribute>
          <xsl:apply-templates />
        </ulink>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>


  <!-- TOC element -->
  <xsl:template match="uci:toc">
    <xsl:message>toc element skipped</xsl:message>
    <xsl:comment> *** explicit TOC dropped *** </xsl:comment>
  </xsl:template>
  
  
  <!-- FORMTEXT element -->
  <xsl:template match="uci:formtext">
    <phrase role="upcast-FORMTEXT">
      <xsl:apply-templates select="descendant::*[not( contains($ucBlockElements, local-name()) )] | descendant::text()" />
    </phrase>
  </xsl:template>
  
  
  <!-- FORMCHECKBOX element -->
  <xsl:template match="uci:formcheckbox">
    <phrase role="upcast-FORMCHECKBOX">
      <xsl:choose>
        <xsl:when test="@uci:user-selection='1'">
          <xsl:text>&#x2713;</xsl:text> <!-- Unicode U+2713: CHECKMARK -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:text>&#x2751;</xsl:text> <!-- Unicode U+2751: LOWER RIGHT SHADOWED SQUARE -->
        </xsl:otherwise>
      </xsl:choose>
    </phrase>
  </xsl:template>
  
  
  <!-- FORMDROPDOWN element -->
  <xsl:template match="uci:formdropdown">
    <xsl:variable name="userchoice" select="@uci:user-selection" />
    <guimenuitem role="upcast-FORMDROPDOWN">
      <xsl:choose> <!-- cater for inconsistent Word output when not in forms editing mode: when current value not defined, use default value. -->
        <xsl:when test="/uci:document/uci:extflow//uci:choicelist[@uci:id=current()/@uci:choicelistref]/uci:choice[@uci:dropdown-index=current()/@uci:user-selection]">
          <xsl:value-of select="/uci:document/uci:extflow//uci:choicelist[@uci:id=current()/@uci:choicelistref]/uci:choice[@uci:dropdown-index=current()/@uci:user-selection]" />
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="/uci:document/uci:extflow//uci:choicelist[@uci:id=current()/@uci:choicelistref]/uci:choice[@uci:dropdown-index=current()/@uci:default-value]" />
        </xsl:otherwise>
      </xsl:choose>
    </guimenuitem>
  </xsl:template>


  <!-- TABLE element -->
  <xsl:template match="uci:block[@uci:type='captioned-table']">
    <xsl:choose>
      <xsl:when test="cals:table">
        <!-- a CALS table -->
        <table>
          <xsl:for-each select="@cals:*">
            <xsl:attribute name="{local-name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
          <caption><xsl:apply-templates select="uci:par" mode="caption"/></caption>
          <xsl:apply-templates select="cals:table/*"/>
        </table>
      </xsl:when>
      <xsl:otherwise>
        <!-- an HTML table -->
        <table>
          <xsl:for-each select="@html:*">
            <xsl:attribute name="{local-name()}"><xsl:value-of select="."/></xsl:attribute>
          </xsl:for-each>
          <caption><xsl:apply-templates select="uci:par" mode="caption"/></caption>
          <xsl:apply-templates select="html:table/*"/>
        </table>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="html:table">
    <informaltable>
      <xsl:for-each select="@html:*">
        <xsl:attribute name="{local-name()}"><xsl:value-of select="."/></xsl:attribute>
      </xsl:for-each>
      <xsl:apply-templates />
    </informaltable>
  </xsl:template>
    
  <xsl:template match="cals:table">
    <informaltable>
      <xsl:for-each select="@cals:*">
        <xsl:attribute name="{local-name()}"><xsl:value-of select="."/></xsl:attribute>
      </xsl:for-each>
      <xsl:apply-templates />
    </informaltable>
  </xsl:template>
  
  
  <!-- CALS table elements -->
  <xsl:template match="cals:*">
    <xsl:element name="{local-name()}">
      <xsl:for-each select="@cals:*">
        <xsl:attribute name="{local-name()}"><xsl:value-of select="."/></xsl:attribute>
      </xsl:for-each>
      
      <!-- for entry: Calculate halign attribute using heuristics: Set only when ALL paragraphs have the same alignment setting! -->
      <xsl:if test="self::cals:entry">
        <xsl:variable name="halign"><xsl:value-of select="child::uci:par[1]/@css:text-align"/></xsl:variable>
        <xsl:choose>
          <xsl:when test="count(child::uci:par) = count(child::uci:par[@css:text-align = $halign]) and count(child::uci:par) &gt; 0">
            <xsl:attribute name="align"><xsl:value-of select="$halign"/></xsl:attribute>
          </xsl:when>
          <xsl:otherwise>
            <!-- do not write the align attribute at all -->
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>
  
  <!-- HTML table elements -->
  <xsl:template match="html:*">
    <xsl:element name="{local-name()}">
      <xsl:for-each select="@html:*">
        <xsl:attribute name="{local-name()}"><xsl:value-of select="."/></xsl:attribute>
      </xsl:for-each>

      <!-- for td/th: Calculate halign attribute using heuristics: Set only when ALL paragraphs have the same alignment setting! -->
      <xsl:if test="self::html:td | self::html:th">
        <xsl:variable name="halign"><xsl:value-of select="child::uci:par[1]/@css:text-align"/></xsl:variable>
        <xsl:choose>
          <xsl:when test="count(child::uci:par) = count(child::uci:par[@css:text-align = $halign]) and count(child::uci:par) &gt; 0">
            <xsl:attribute name="align"><xsl:value-of select="$halign"/></xsl:attribute>
          </xsl:when>
          <xsl:otherwise>
            <!-- do not write the align attribute at all -->
          </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="@css:vertical-align">
          <xsl:attribute name="valign" select="@css:vertical-align"/>
        </xsl:if>
      </xsl:if>

      <xsl:apply-templates />
    </xsl:element>
  </xsl:template>
  
  <!-- nested CALS TABLE element -->
  <xsl:template match="cals:entry/cals:table">
    <xsl:comment> ### complete table dropped: nested tables not supported in CALS (use HTML table model!) ### </xsl:comment>
    <xsl:message terminate="yes">nested tables not supported in CALS. Use HTML table model!</xsl:message>
  </xsl:template>


  <!-- COLUMNBREAK element -->
  <xsl:template match="uci:columnbreak">
    <xsl:comment> ***columnbreak in original document*** </xsl:comment>
  </xsl:template>


  <!-- PAGEBREAK element -->
  <xsl:template match="uci:pagebreak" name="pagebreak">
    <xsl:comment> ***pagebreak in original document*** </xsl:comment>
  </xsl:template>
  
  
  <!-- LINEBREAK element -->
  <xsl:template match="uci:linebreak">
    <!-- <xsl:text>&#x2028;</xsl:text> --> <!-- Unicode explicit line break character - should we use it? -->
    <xsl:text>&#xa;</xsl:text> <!-- for now, write simple newline character -->
  </xsl:template>


  <!-- OLE element - suppress! -->
  <xsl:template match="ole">
  </xsl:template>

  <!-- OBJECT element -->
  <xsl:template match="uci:object[@uci:type='math']">
  	<inlineequation>
  		<xsl:choose>
  		<xsl:when test="mml:math">
  			<xsl:apply-templates select="mml:math" />
  		</xsl:when>
  		<xsl:otherwise>
  			<xsl:apply-templates select="uci:image" />
  		</xsl:otherwise>
  		</xsl:choose>
  	</inlineequation>
  </xsl:template>
  
  <xsl:template match="uci:object">
  	<xsl:apply-templates select="uci:image" />
  </xsl:template>
  
  
  <!-- MathML elements: copy! -->
    <xsl:template match="*[namespace-uri(.)='http://www.w3.org/1998/Math/MathML']">
    <xsl:copy>
      <xsl:apply-templates />
    </xsl:copy>
  </xsl:template>


<!-- ========== Named templates ===================================================================== -->
  <!-- Create the lang attribute if present in the original -->
  <xsl:template name="lang-attr">
    <xsl:if test="@xml:lang">
      <xsl:attribute name="lang">
        <xsl:value-of select="@xml:lang" />
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

  <!-- Create the id attribute if present in the original -->
  <xsl:template name="id-attr">
    <xsl:if test="@uci:id">
      <xsl:attribute name="id">
        <xsl:value-of select="@uci:id" />
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

<!-- ========== Functions ===================================================================== -->
  <xsl:function name="ilf:quoteForAttribute">
    <xsl:param name="s"/>
    <xsl:value-of select="replace($s, '([^&quot;]*)&quot;([^&quot;]*)', '$1&amp;quot;$2')"/>
  </xsl:function>

<!-- ========== Error handling ===================================================================== -->

  <!-- SUBDOC element -->
  <xsl:template match="uci:subdoc">
    <xsl:message>unexpected subdoc element found - report to &gt;support@infinity-loop.de&lt;</xsl:message>
  </xsl:template>
  
</xsl:stylesheet>
