<xsl:stylesheet version="2.0" 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xlink="http://www.w3.org/1999/xlink" 
  xmlns:html="http://www.w3.org/HTML/1998/html4"
  xmlns:cals="http://www.infinity-loop.de/namespace/2006/upcast-cals"
  xmlns:mml="http://www.w3.org/1998/Math/MathML"
  xmlns:ilext="java:de.infinityloop.util.xslt.ExtensionFunctions"
  xmlns:ilf="http://www.infinity-loop.de/namespace/project/xslfunctions"
  xmlns:xd="http://www.pnp-software.com/XSLTdoc"
  xmlns:uci="http://www.infinity-loop.de/namespace/2006/upcast-internal"
  xmlns:saxon="http://saxon.sf.net/"
  xmlns:css="http://www.infinity-loop.de/namespace/2006/upcast-css"
  xmlns:csso="http://www.infinity-loop.de/namespace/2006/upcast-cssoverride"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:db="http://www.infinity-loop.de/namespace/internal/docbook45"
  exclude-result-prefixes="xd ilext uci ilf saxon css csso xlink xs html db mml cals" >

<!-- **************************************************
     XSLT stylesheet to transform a UPL-pre-processed upCast document tree
                     to DocBook 4.5.
                     
                     Root element: book
                     
                     
     DISCLAIMER:
       This stylesheet is provided AS IS with no warranty.
       Use completely at your own risk!
                       
     ******

     Author :  Christian Roth <roth@infinity-loop.de>
     Version:  2 (pre-2.x versions were used in upCast 5.x)
     
     ******
     ***************************************************
-->

  <!-- Core DocBook 4.5 conversion templates. These are overridden by the templates in this specific stylesheet where needed. -->
  <xsl:import href="uc2db45-core.xsl"/>
  

  <xsl:param name="missingChapterTitleText">### missing &lt;chapter> title ###</xsl:param>
  <xsl:param name="missingPartTitleText">### missing &lt;part> title ###</xsl:param>
  <xsl:param name="prefaceTitleText">Preface</xsl:param>


  <!-- DOCUMENT element -->
  <xsl:template match="uci:document">
    <book>
      <xsl:call-template name="id-attr"/>
      <xsl:call-template name="lang-attr"/>
      
      <title>
        <xsl:choose>
          <xsl:when test="uci:body/uci:par[@uci:rawclass='Title']">
            <xsl:apply-templates select="uci:body/uci:par[@uci:rawclass='Title'][1]" />
          </xsl:when>
          <xsl:otherwise>
            <xsl:apply-templates select="uci:head/uci:documentinfo/uci:property[@uci:name='title']/@uci:value" />
          </xsl:otherwise>
        </xsl:choose>
      </title>
      <xsl:apply-templates select="uci:head"/>
      <xsl:choose>
        <xsl:when test="not(uci:body/uci:section)">
          <!-- handle special case where there are no sections at all: create an artificial chapter. -->
          <chapter>
            <title><xsl:value-of select="$missingChapterTitleText"/></title>
            <xsl:apply-templates select="uci:body/* except uci:body/uci:par[@uci:rawclass='Title'][1]"/>
          </chapter>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="uci:body/* except uci:body/uci:par[@uci:rawclass='Title'][1]"/>
        </xsl:otherwise>
      </xsl:choose>
      <!-- We do not process uci:extflow here! Data from this part is fetched explicitly when needed. -->
    </book>
  </xsl:template>

  <!-- DOCUMENTINFO element -->
  <xsl:template match="uci:documentinfo">
    <bookinfo>
      <xsl:call-template name="id-attr"/>
      <xsl:call-template name="lang-attr"/>
      <xsl:choose>
        <xsl:when test="uci:property[@uci:name='title']">
          <title><xsl:value-of select="uci:property[@uci:name='title']/@uci:value"/></title>
        </xsl:when>
        <xsl:when test="/uci:document/uci:body/uci:par[@uci:rawclass='Title']">
          <title><xsl:value-of select="/uci:document/uci:body/uci:par[@uci:rawclass='Title']"/></title>
        </xsl:when>
        <xsl:otherwise>
          <title>Untitled Book</title>
        </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="uci:property[not(@uci:name='title')]"/>
    </bookinfo>
  </xsl:template>
  
  
  <!-- SECTIONINTRO -->
  
  <!-- This is the content _before_ the first section. We group it into the preface element. -->
  <xsl:template match="uci:sectionintro[@uci:level=0]">
    <xsl:choose>
      <xsl:when test="*[local-name()!='part']">
        <preface>
          <title><xsl:value-of select="$prefaceTitleText"/></title>
          <xsl:apply-templates/>
        </preface>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <!-- Other sectionintros, not top-level: discard -->
  <xsl:template match="uci:sectionintro[@level!=0]">
    <xsl:apply-templates/>
  </xsl:template>

  
  <!-- SECTION element level 1 = part -->
  <xsl:template match="uci:section[@uci:level='1']">
    <xsl:choose>
      <xsl:when test="not(following-sibling::*) and count(*[not(self::uci_section)]) gt 0">
        <!-- this is the only section of level 1 and it has no heading: discard it -->
        <xsl:apply-templates />
      </xsl:when>
      <xsl:otherwise>
        <part>
          <title><xsl:apply-templates select="uci:par[@uci:heading-level=current()/@uci:level]/node()" /></title>
          <xsl:apply-templates select="*[self::uci:section or not(@uci:heading-level) or (@uci:heading-level ne current()/@uci:level)]" />
        </part>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
    
  <!-- SECTION element level 2 = chapter -->
  <xsl:template match="uci:section[@uci:level='2']">
    <chapter>
      <title><xsl:apply-templates select="uci:par[@uci:heading-level=current()/@uci:level]/node()" /></title>
      <xsl:choose>
        <xsl:when test="*[self::uci:section or not(@uci:heading-level) or (@uci:heading-level ne current()/@uci:level)]">
          <xsl:apply-templates select="*[self::uci:section or not(@uci:heading-level) or (@uci:heading-level ne current()/@uci:level)]" />
        </xsl:when>
        <xsl:otherwise>
          <!-- write empty paragraph to be valid -->
          <para><xsl:comment>empty para to satisfy content model</xsl:comment></para>
        </xsl:otherwise>
      </xsl:choose>
    </chapter>
  </xsl:template>

</xsl:stylesheet>
